#!/bin/bash

# Pour récupérer toutes les signatures des mails dans le dossier
# idoine stocké sur zamok

# NB : si tu n'as pas "cadre", bah t'es puni, remplace par echo

# -------

# On lance l'agent gpg avec un pinentry commandline
cadre "Starting gpg-agent"
eval $(gpg-agent --daemon --pinentry-program /usr/bin/pinentry-curses)
echo $GPG_AGENT_INFO
agentpid=$(echo $GPG_AGENT_INFO | cut -d: -f 2)

# On récupère les mails de signatures
where=`mktemp -d`
cadre "Storing mails copied in ${where}"

# On récupère la liste des fichiers qui n'ont pas de S (\Seen) dans leurs flags
# (on vire le \x0 qui est à la fin, et on prie pour qu'il n'y ait jamais d'espaces dans les noms de mails)
MAILS=$(ssh zamok find "~/Mail/.Signatures/cur/" -type f -regex "'[^:]+:2,[^S]*'" -print0 | sed "s/\x0/ /g;s/ $//")
# On parsème de virgules et de backslash là où il faut et on les rappatrie
echo $MAILS | sed 's/,/\\\\,/g;s/ /,/g' | xargs -I % scp zamok:{%} ${where}

# On traite les mails un par un
cadre "Decrypting and importing signatures"
for mail in ${where}/*;
do
    gpg2 --decrypt ${mail} | gpg2 --import;
done

# On marque (peut-être) les mails distants comme lus
cadre "Cleaning up"
echo -n "Mark distant e-mails as read ? [Y/n]"
read -r ans
# On ajoute le flag S (\Seen) aux mails concernés là où il faut dans l'ordre alphabétique
# en ne touchant pas au reste de leur nom
# Attention au "''" et aux \$ parce que ssh needs moar escapingz
[[ "${ans}" = "n" ]] || [[ "${ans}" = "N" ]] || ssh zamok rename -v "'s/^([^:]+):2,([A-R]*)([^S]*)$/\$1:2,\$2S\$3/'" $MAILS

# On fait le ménage
echo -n "Remove local tempfolder ? [Y/n]"
read -r ans
[[ "${ans}" = "n" ]] || [[ "${ans}" = "N" ]] || rm -rfv ${where}

cadre "Sending freshly-signed key to servers"
echo -n "Sure ? [Y/n]"
read -r ans
[[ "${ans}" = "n" ]] || [[ "${ans}" = "N" ]] || gpg --send-keys A94025B0
